# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.
# @author ChrisJaunes

from metakernel import Magic, option
from IPython.display import IFrame, Javascript


class BlocklyMagic(Magic):

    @option(
        '-o', '--page_from_origin', action='store', default=None,
        help='Load remote page about blockly'
    )
    @option(
        '-l', '--page_from_local', action='store', default=None,
        help='Load local page about blockly'
    )
    @option(
        '-t', '--template_data', action='store', default=None,
        help='generate page based on template and load, must be used with parameters(-o or -l)'
     )
    @option(
        '-h', '--height', action='store', default=350,
        help='set height of iframe '
    )
    def line_blockly(self, page_from_origin=None, page_from_local=None, template_data=None, height=350):
        """
        %blockly - show visual code

        This line magic will allow visual code editing
        If both -o and -l are provided, only -l is used
        
        Examples:
            %blockly --page_from_origin http://host[:port]/blockly_page.html
            %blockly --page_from_local blockly_page.html
            %blockly --page_from_origin http://host[:port]/blockly_template.html --template_data template_data
            %blockly --height 600
        """
        # Display iframe:
        script = """
        if(document.receiveBlocklyPythonCode === undefined) {
            document.receiveBlocklyPythonCode = function ( event ) {
                IPython.notebook.insert_cell_above().set_text(event.data);
            }
            window.addEventListener("message", document.receiveBlocklyPythonCode, false)
        }
        """
        #print(script)
        self.kernel.Display(Javascript(script))

        if height is None:
            height = 350
        if template_data is not None:
            if page_from_local is not None:
                with open(html_from_local, "rb") as fp:
                    html_template = fp.read().decode("utf-8")
            elif page_from_origin is not None:
                try:
                    import urllib.request
                    urlopen = urllib.request.urlopen
                except:  # python2
                    import urllib
                    urlopen = urllib.urlopen
                page_template = urlopen(page_from_origin).read().decode("utf-8")
            else:
                raise ValueError("No -l or -o is provided")
            with open(template_data+'-toolbox.xml', "rb") as fp:
                blockly_toolbox = fp.read().decode("utf-8")
            html_template = html_template.replace("MY_BLOCKLY_TOOLBOX", blockly_toolbox)
            with open(template_data + '-workspace.xml', "rb") as fp:
                blockly_workspace = fp.read().decode("utf-8")
            html_template = html_template.replace("MY_BLOCKLY_WORKSPACE", blockly_workspace)
            with open(template_data + '-blocks.js', "rb") as fp:
                blockly_blocks = fp.read().decode("utf-8")
            html_template = html_template.replace("MY_BLOCKLY_BLOCKS_JS", blockly_blocks)
            with open(template_data + '.html', 'w') as fp:
                fp.write(html_template)
            page_from_local = template_data + '.html'
        if page_from_local is not None:
            self.kernel.Display(IFrame(page_from_local, width='100%', height=height))
        elif page_from_origin is not None:
            self.kernel.Display(IFrame(page_from_origin, width='100%', height=height))
        else:
            self.kernel.Display(IFrame("https://developers-dot-devsite-v2-prod.appspot.com/blockly/blockly-demo/blockly-demo", width='100%', height=height))


def register_magics(kernel):
    kernel.register_magics(Magic)


def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_line_magic
    kernel = IPythonKernel()
    magic = BlocklyMagic(kernel)
    # Make magics callable:
    kernel.line_magics["blockly"] = magic

    @register_line_magic
    def blockly(line):
        """
        Use the blockly code visualizer and generator.
        """
        kernel.call_magic("%blockly " + line)



import platform
import unittest
import re
import os
import sys

from metakernel import pexpect, replwrap
from metakernel.tests.utils import get_log, get_log_text

class REPLWrapTestCase(unittest.TestCase):

    def setUp(self):
        super(REPLWrapTestCase, self).setUp()
        self.save_ps1 = os.getenv('PS1', r'\$')
        self.save_ps2 = os.getenv('PS2', '>')
        os.putenv('PS1', r'\$')
        os.putenv('PS2', '>')

    def tearDown(self):
        super(REPLWrapTestCase, self).tearDown()
        os.putenv('PS1', self.save_ps1)
        os.putenv('PS2', self.save_ps2)

    def test_bash(self):
        bash = replwrap.bash()
        res = bash.run_command("time")
        assert 'real' in res, res

        # PAGER should be set to cat, otherwise man hangs
        res = bash.run_command('man sleep', timeout=2)
        assert 'SLEEP' in res, res

        # should handle CR by default
        cmd = r'for i in {1..3};do echo -ne "\r$i"; sleep 1; done'
        res = bash.run_command(cmd)
        assert '\r1\r2\r3' in res

        # should handle CRs in a stream
        res = bash.run_command(cmd, stream_handler=sys.stdout.write)
        assert res == ''

        # should handle lines with a line handler
        logger = get_log()
        res = bash.run_command('echo "1\n2\n3"', line_handler=logger.info)
        assert res == ''
        text = get_log_text(logger)
        assert '1\n2\n3' in text

    def test_multiline(self):
        bash = replwrap.bash()
        res = bash.run_command("echo '1 2\n3 4'")
        self.assertEqual(res.strip().splitlines(), ['1 2', '3 4'])

        # Should raise ValueError if input is incomplete
        try:
            bash.run_command("echo '5 6")
        except ValueError:
            pass
        else:
            assert False, "Didn't raise ValueError for incomplete input"

        # Check that the REPL was reset (SIGINT) after the incomplete input
        res = bash.run_command("echo '1 2\n3 4'")
        self.assertEqual(res.strip().splitlines(), ['1 2', '3 4'])

    def test_existing_spawn(self):
        child = pexpect.spawnu("bash", timeout=5, echo=False)
        repl = replwrap.REPLWrapper(child, re.compile('[$#]'),
                                    "PS1='{0}' PS2='{1}' "
                                    "PROMPT_COMMAND='' TERM='dumb'")

        res = repl.run_command("echo $HOME")
        assert res.startswith('/'), res

    def test_python(self):
        if platform.python_implementation() == 'PyPy':
            raise unittest.SkipTest("This test fails on PyPy because of REPL differences")

        if platform.system() == 'Darwin':
            raise unittest.SkipTest("This test fails on macOS because of REPL differences")

        p = replwrap.python(sys.executable)
        res = p.run_command('4+7')
        assert res.strip() == '11'

        res = p.run_command('for a in range(3): print(a)\n')
        assert res.strip().splitlines() == ['0', '1', '2']

    def test_bracketed_paste(self):
        # Readline paste bracketing is easily toggled in bash, but can be harder elsewhere
        # This tests that run_command() still works with it enabled (the default for readline,
        # but overriden by bash and python)
        bash = replwrap.bash()
        bash.run_command("bind 'set enable-bracketed-paste on'")
        res = bash.run_command("echo '1 2\n3 4'")
        self.assertEqual(res.strip().splitlines(), ['1 2', '3 4'])


if __name__ == '__main__':
    unittest.main()



import shutil

from metakernel.tests.utils import (get_kernel, get_log_text, 
                                    clear_log_text, EvalKernel)

import pytest

NO_DOT = shutil.which("dot") is None

@pytest.mark.skipif(NO_DOT, reason="Requires dot from graphviz")
def test_dot_magic_cell():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("""%%dot

graph A { a->b };
""")

    text = get_log_text(kernel)
    assert "Display Data" in text, text

@pytest.mark.skipif(NO_DOT, reason="Requires dot from graphviz")
def test_dot_magic_line():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("%dot graph A { a->b };")

    text = get_log_text(kernel)
    assert "Display Data" in text, text



"""
Test the metakernel_python kernel using jupyter_kernel_test with supported capabilities
"""
import unittest
import jupyter_kernel_test as jkt

class MetaKernelPythonTests(jkt.KernelTests):

    # REQUIRED

    # the kernel to be tested
    # this is the normally the name of the directory containing the
    # kernel.json file - you should be able to do
    # `jupyter console --kernel KERNEL_NAME`
    kernel_name = "metakernel_python"

    # Everything else is OPTIONAL

    # the name of the language the kernel executes
    # checked against language_info.name in kernel_info_reply
    language_name = "python"

    # the normal file extension (including the leading dot) for this language
    # checked against language_info.file_extension in kernel_info_reply
    file_extension = ".py"

    # code which should write the exact string `hello, world` to STDOUT
    code_hello_world = "print('hello, world')"

    # code which should cause (any) text to be written to STDERR
    code_stderr = "import sys; print('test', file=sys.stderr)"

    # samples for the autocompletion functionality
    # for each dictionary, `text` is the input to try and complete, and
    # `matches` the list of all complete matching strings which should be found
    completion_samples = [
        {
            'text': 'zi',
            'matches': {'zip'},
        },
    ]

    # samples for testing code-completeness (used by console only)
    # these samples should respectively be unambigiously complete statements
    # (which should be executed on <enter>), incomplete statements or code
    # which should be identified as invalid
    complete_code_samples = ['1', "print('hello, world')", "def f(x):\n  return x*2\n\n\n"]
    incomplete_code_samples = ["print('''hello", "def f(x):\n  x*2"]
    invalid_code_samples = ['import = 7q']

    # code which should cause a help pager to be displayed (as of 4.1, this is
    # displayed by the notebook only as inline text, so it's probably more
    # useful for console clients)
    code_page_something = "zip?"

    # code which should generate a (user-level) error in the kernel, and send
    # a traceback to the client
    code_generate_error = "raise"

    # a statement or block of code which generates a result (which is shown
    # as Out[n] instead of just something printed to stdout)
    # running each `code` should cause `result` to be displayed (note that the
    # result here will always be a string representation of whatever the actual
    # result type is - be careful of string formatting)
    code_execute_result = [
        {'code': "1+2+3", 'result': "6"},
        {'code': "[n*n for n in range(1, 4)]", 'result': "[1, 4, 9]"}
    ]

    # code which generates some sort of rich output
    # for each `code` input a single rich display object with the specified
    # `mime` type should be sent to the frontend
    # note that this expects a `display_data` message rather than
    # `execute_result`; this test might be a little too inflexible in some cases
    code_display_data = [
        {'code': "from IPython.display import HTML, display; display(HTML('<b>test</b>'))",
         'mime': "text/html"},
        {'code': "from IPython.display import Math, display; display(Math('\\frac{1}{2}'))",
         'mime': "text/latex"}
    ]

    # test the support for object inspection
    # the sample should be a name about which the kernel can give some help
    # information (a built-in function is probably a good choice)
    # only the default inspection level (equivalent to ipython "obj?")
    # is currently tested
    code_inspect_sample = "zip"

if __name__ == '__main__':
    unittest.main()
